import gzip
from typing import Tuple, Optional
from xml.etree import ElementTree as ET

import pandas as pd


def load_and_parse_savegame(
    file_path: str,
) -> Tuple[Optional[ET.Element], Optional[pd.DataFrame]]:
    """
    Loads, decompresses, and parses a save game file.

    Args:
        file_path: The path to the .xml.gz file.

    Returns:
        A tuple containing the XML root element for the tree view
        and a pandas DataFrame for the plot. Returns (None, None) on error.
    """
    try:
        with gzip.open(file_path, "rb") as f:
            content = f.read().decode("utf-8")
            root = ET.fromstring(content)

            data = []
            trade_entries_root = ET.Element("trades")  # Create a root for the snippet
            for entry in root.findall(".//entries[@type='trade']"):
                for log in entry.findall(".//log"):
                    trade_entries_root.append(log)
                    data.append(log.attrib)

            df = pd.DataFrame(data)
            return trade_entries_root, df

    except (FileNotFoundError, gzip.BadGzipFile, ET.ParseError) as e:
        print(f"Error processing file: {e}")
        return None, None
