import pandas as pd
import pyqtgraph as pg
from PySide6 import QtWidgets
from PySide6.QtCore import Slot

from . import data_processing
from .utils import load_image


class MainWindow(QtWidgets.QMainWindow):
    def __init__(self):
        super().__init__()
        self.init_ui()

    def init_ui(self):
        self.setWindowTitle("nullfinder")
        self.setWindowIcon(load_image("assets/icon.png", "nullfinder"))
        central_widget = QtWidgets.QWidget()
        self.setCentralWidget(central_widget)
        layout = QtWidgets.QVBoxLayout(central_widget)
        self.label_selected_file = QtWidgets.QLabel("No file selected.")
        layout.addWidget(self.label_selected_file)
        button_open = QtWidgets.QPushButton("Open Save Game (*.xml.gz)")
        button_open.setShortcut("Ctrl+O")
        button_open.clicked.connect(self.open_file_dialog)
        layout.addWidget(button_open)
        horizontal_layout = QtWidgets.QHBoxLayout()
        layout.addLayout(horizontal_layout)
        self.tree_widget = QtWidgets.QTreeWidget()
        self.tree_widget.setHeaderLabel("XML Structure")
        horizontal_layout.addWidget(self.tree_widget)
        self.plot_widget = pg.PlotWidget()
        horizontal_layout.addWidget(self.plot_widget)
        button_exit = QtWidgets.QPushButton("Exit")
        button_exit.clicked.connect(self.close)
        layout.addWidget(button_exit)

    @Slot()
    def open_file_dialog(self):
        file_dialog = QtWidgets.QFileDialog(self)
        file_dialog.setNameFilter("Save Game Files (*.xml.gz)")
        if file_dialog.exec():
            selected_files = file_dialog.selectedFiles()
            if selected_files:
                self.label_selected_file.setText(
                    f"Selected file: {selected_files[0]}")
                self.process_file(selected_files[0])
            else:
                self.label_selected_file.setText("No file selected.")

    def process_file(self, file_path):
        trade_root, df = data_processing.load_and_parse_savegame(file_path)

        if trade_root is None or df is None:
            QtWidgets.QMessageBox.critical(self, "Error",
                                           "Failed to process file.")
            return

        self.update_tree_widget(trade_root)
        self.update_plot_widget(df)

    def update_tree_widget(self, root_element):
        self.tree_widget.clear()
        self.populate_tree(root_element, self.tree_widget)

    def update_plot_widget(self, df):
        self.plot_widget.clear()
        if not df.empty and "time" in df.columns and "price" in df.columns:
            try:
                time_vals = pd.to_numeric(df["time"], errors="coerce")
                price_vals = pd.to_numeric(df["price"], errors="coerce")
                valid_data = pd.DataFrame(
                    {"time": time_vals, "price": price_vals}
                ).dropna()

                self.plot_widget.plot(
                    valid_data["time"],
                    valid_data["price"],
                    pen=None,
                    symbol="o",
                    symbolSize=5,
                )
            except Exception as e:
                QtWidgets.QMessageBox.warning(
                    self, "Plotting Error", f"Could not plot data: {e}"
                )

    def populate_tree(self, element, parent):
        item = QtWidgets.QTreeWidgetItem(parent)
        item.setText(0, element.tag)
        if element.text and element.text.strip():
            text_item = QtWidgets.QTreeWidgetItem(item)
            text_item.setText(0, element.text.strip())
        for attr_name, attr_value in element.attrib.items():
            attr_item = QtWidgets.QTreeWidgetItem(item)
            attr_item.setText(0, f"{attr_name}: {attr_value}")
        for child in element:
            self.populate_tree(child, item)
